@@ -22,11 +22,11 @@
  * @title: zxing
  *
  * Detect bar codes in the video streams and send them as element messages to
- * the #GstBus if .#GstZXing:message property is %TRUE.
- * If the .#GstZXing:attach-frame property is %TRUE, the posted barcode message
+ * the #GstBus if #GstZXing:message property is %TRUE.
+ * If the #GstZXing:attach-frame property is %TRUE, the posted barcode message
  * includes a sample of the frame where the barcode was detected (Since 1.18).
  *
- * The element generate messages named `barcode`. The structure contains these fields:
+ * The element generates messages named `barcode`. The structure contains these fields:
  *
  * * #GstClockTime `timestamp`: the timestamp of the buffer that triggered the message.
  * * gchar * `type`: the symbol type.
@@ -37,16 +37,18 @@
  *   the .#GstZXing:attach-frame property was set to %TRUE (Since 1.18)
  *
  *   This element is based on the c++ implementation of zxing which can found
- *   at https://github.com/nu-book/zxing-cpp.
+ *   at <https://github.com/zxing-cpp/zxing-cpp>.
  *
  * ## Example launch lines
  * |[
  * gst-launch-1.0 -m v4l2src ! videoconvert ! zxing ! videoconvert ! xvimagesink
  * ]| This pipeline will detect barcodes and send them as messages.
  * |[
- * gst-launch-1.0 -m v4l2src ! tee name=t ! queue ! videoconvert ! zxing ! fakesink t. ! queue ! xvimagesink
+ * gst-launch-1.0 -m v4l2src ! tee name=t ! queue ! videoconvert ! zxing ! fakesink t. ! queue ! videoconvert ! xvimagesink
  * ]| Same as above, but running the filter on a branch to keep the display in color
  *
+ * Since: 1.18
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,9 +62,12 @@
 
 #include <gst/video/video.h>
 
-#include "ReadBarcode.h"
-#include "TextUtfEncoding.h"
-#include "ZXVersion.h"
+#if __has_include("ZXing/WriteBarcode.h") /* only available in version >= 3.x */
+#include "ZXing/ZXingCpp.h" /* future proof one-stop-shop header */
+#else
+#include "ZXing/ReadBarcode.h"
+#include "ZXing/ZXVersion.h"
+#endif
 
 using namespace ZXing;
 
@@ -84,7 +89,31 @@ enum
   PROP_FORMAT,
 };
 
-enum
+/**
+ * GstBarCodeFormat:
+ *
+ * @BARCODE_FORMAT_ALL all format
+ * @BARCODE_FORMAT_AZTEC aztec format
+ * @BARCODE_FORMAT_CODABAR codabar format
+ * @BARCODE_FORMAT_CODE_39 code39 format
+ * @BARCODE_FORMAT_CODE_93 code 93 format
+ * @BARCODE_FORMAT_CODE_128 code 128 format
+ * @BARCODE_FORMAT_DATA_MATRIX data matrix format
+ * @BARCODE_FORMAT_EAN_8 EAN 8 format
+ * @BARCODE_FORMAT_EAN_13 EAN 13 format
+ * @BARCODE_FORMAT_ITF ITF format
+ * @BARCODE_FORMAT_MAXICODE maxicode format
+ * @BARCODE_FORMAT_PDF_417 pdf 417 format
+ * @BARCODE_FORMAT_QR_CODE QR code format
+ * @BARCODE_FORMAT_RSS_14 RSS 14 format
+ * @BARCODE_FORMAT_RSS_EXPANDED RSS expanded format
+ * @BARCODE_FORMAT_UPC_A UPC_A format
+ * @BARCODE_FORMAT_UPC_E UPC_E format
+ * @BARCODE_FORMAT_UPC_EAN_EXTENSION UPC_EAN_EXTENSION format
+ *
+ * Since: 1.18
+ */
+typedef enum
 {
   BARCODE_FORMAT_ALL,
   BARCODE_FORMAT_AZTEC,
@@ -104,7 +133,7 @@ enum
   BARCODE_FORMAT_UPC_A,
   BARCODE_FORMAT_UPC_E,
   BARCODE_FORMAT_UPC_EAN_EXTENSION
-};
+} GstBarCodeFormat;
 
 static const GEnumValue barcode_formats[] = {
   {BARCODE_FORMAT_ALL, "ALL", "all"},
@@ -113,7 +142,7 @@ static const GEnumValue barcode_formats[] = {
   {BARCODE_FORMAT_CODE_39, "CODE_39", "code_39"},
   {BARCODE_FORMAT_CODE_93, "CODE_93", "code_93"},
   {BARCODE_FORMAT_CODE_128, "CODE_128", "code_128"},
-  {BARCODE_FORMAT_DATA_MATRIX, "PNG", "png"},
+  {BARCODE_FORMAT_DATA_MATRIX, "DATA_MATRIX", "data_matrix"},
   {BARCODE_FORMAT_EAN_8, "EAN_8", "ean_8"},
   {BARCODE_FORMAT_EAN_13, "EAN_13", "ean_13"},
   {BARCODE_FORMAT_ITF, "ITF", "itf"},
@@ -175,7 +204,7 @@ struct _GstZXing
   gboolean rotate;
   gboolean faster;
   ImageFormat image_format;
-  guint barcode_format;
+  GstBarCodeFormat barcode_format;
 };
 
 static void gst_zxing_set_property (GObject * object, guint prop_id,
@@ -208,27 +237,60 @@ gst_zxing_class_init (GstZXingClass * g_class)
   gobject_class->set_property = gst_zxing_set_property;
   gobject_class->get_property = gst_zxing_get_property;
 
+  /**
+   * GstZXing::message:
+   *
+   * Post a barcode message for each detected code.
+   *
+   * Since: 1.18
+   */
   g_object_class_install_property (gobject_class, PROP_MESSAGE,
       g_param_spec_boolean ("message",
           "message", "Post a barcode message for each detected code",
           DEFAULT_MESSAGE,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
+  /**
+   * GstZXing::attach-frame:
+   *
+   * Attach a frame dump to each barcode message
+   *
+   * Since: 1.18
+   */
   g_object_class_install_property (gobject_class, PROP_ATTACH_FRAME,
       g_param_spec_boolean ("attach-frame", "Attach frame",
           "Attach a frame dump to each barcode message",
           DEFAULT_ATTACH_FRAME,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
-
+  /**
+   * GstZXing::try-rotate:
+   *
+   * Try to rotate the frame to detect barcode (slower)
+   *
+   * Since: 1.18
+   */
   g_object_class_install_property (gobject_class, PROP_TRY_ROTATE,
       g_param_spec_boolean ("try-rotate", "Try rotate",
           "Try to rotate the frame to detect barcode (slower)",
           DEFAULT_TRY_ROTATE,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+  /**
+   * GstZXing::try-faster:
+   *
+   * Try faster to analyze the frame
+   *
+   * Since: 1.18
+   */
   g_object_class_install_property (gobject_class, PROP_TRY_FASTER,
       g_param_spec_boolean ("try-faster", "Try faster",
           "Try faster to analyze the frame", DEFAULT_TRY_FASTER,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+  /**
+   * GstZXing::format:
+   *
+   * Barcode image format
+   *
+   * Since: 1.18
+   */
   g_object_class_install_property (gobject_class, PROP_FORMAT,
       g_param_spec_enum ("format", "barcode format", "Barcode image format",
           GST_TYPE_BARCODE_FORMAT, BARCODE_FORMAT_ALL,
@@ -248,6 +310,8 @@ gst_zxing_class_init (GstZXingClass * g_class)
       GST_DEBUG_FUNCPTR (gst_zxing_transform_frame_ip);
   vfilter_class->set_info =
       GST_DEBUG_FUNCPTR (gst_zxing_set_info);
+
+  gst_type_mark_as_plugin_api (GST_TYPE_BARCODE_FORMAT, (GstPluginAPIFlags)0);
 }
 
 static void
@@ -284,7 +348,7 @@ gst_zxing_set_property (GObject * object, guint prop_id, const GValue * value,
       zxing->faster = g_value_get_boolean (value);
       break;
     case PROP_FORMAT:
-      zxing->barcode_format = g_value_get_enum (value);
+      zxing->barcode_format = (GstBarCodeFormat)g_value_get_enum (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -331,7 +395,11 @@ gst_zxing_set_info (GstVideoFilter * vfilter, GstCaps * in,
   switch (in_info->finfo->format) {
     case GST_VIDEO_FORMAT_ARGB:
     case GST_VIDEO_FORMAT_xRGB:
+  #if ZXING_VERSION_MAJOR >= 3
+      zxing->image_format = ImageFormat::ARGB;
+  #else
       zxing->image_format = ImageFormat::XRGB;
+  #endif
       break;
     case GST_VIDEO_FORMAT_Y444:
     case GST_VIDEO_FORMAT_Y42B:
@@ -354,11 +422,21 @@ gst_zxing_transform_frame_ip (GstVideoFilter * vfilter, GstVideoFrame * frame)
   GstZXing *zxing = GST_ZXING (vfilter);
   gpointer data;
   gint height, width;
-  DecodeHints hints;
+  #if ZXING_VERSION_MAJOR >= 3 || \
+      (ZXING_VERSION_MAJOR >= 2 && ZXING_VERSION_MINOR >= 2)
+    ReaderOptions hints;
+  #else
+    DecodeHints hints;
+  #endif
 
   hints.setTryRotate(zxing->rotate);
   hints.setTryHarder(!zxing->faster);
-  hints.setFormats(BarcodeFormatFromString (barcode_formats[zxing->barcode_format].value_name));
+  try {
+    hints.setFormats(BarcodeFormatFromString (barcode_formats[zxing->barcode_format].value_name));
+  } catch (const std::invalid_argument&) {
+    /* since 3.0 zxing-cpp throws if BarcodeFormatFromString() can't parse the string.
+     * see https://gitlab.freedesktop.org/gstreamer/gstreamer/-/issues/4893#note_3325953 */
+  }
 
   /* all formats we support start with an 8-bit Y plane. zxing doesn't need
    * to know about the chroma plane(s) */
