@@ -341,13 +341,14 @@ gst_frei0r_src_class_init (GstFrei0rSrcClass * klass,
   if (desc == NULL || *desc == '\0')
     desc = "No details";
   gst_element_class_set_metadata (gstelement_class, class_data->info.name,
-      "Src/Video", desc, author);
+      "Source/Video", desc, author);
   g_free (author);
 
   caps = gst_frei0r_caps_from_color_model (class_data->info.color_model);
 
   templ = gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS, caps);
   gst_element_class_add_pad_template (gstelement_class, templ);
+  gst_caps_unref (caps);
 
   gstbasesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_frei0r_src_set_caps);
   gstbasesrc_class->is_seekable =
