@@ -109,7 +109,6 @@ struct _GstV4l2CodecH265Dec
   GstV4l2CodecAllocator *src_allocator;
   GstV4l2CodecPool *src_pool;
   gint min_pool_size;
-  gboolean has_videometa;
   gboolean streaming;
   gboolean copy_frames;
   gboolean need_sequence;
@@ -118,6 +117,8 @@ struct _GstV4l2CodecH265Dec
   struct v4l2_ctrl_hevc_pps pps;
   struct v4l2_ctrl_hevc_scaling_matrix scaling_matrix;
   struct v4l2_ctrl_hevc_decode_params decode_params;
+  GArray *ext_sps_st_rps;
+  GArray *ext_sps_lt_rps;
   GArray *slice_params;
   GArray *entry_point_offsets;
 
@@ -130,6 +131,7 @@ struct _GstV4l2CodecH265Dec
   gboolean support_scaling_matrix;
   gboolean support_slice_parameters;
   gboolean support_entry_point_offsets;
+  gboolean support_long_short_term_rps;
 
   GstVideoConverter *convert;
   gboolean need_crop;
@@ -197,6 +199,14 @@ gst_v4l2_decoder_h265_api_check (GstV4l2Decoder * decoder)
       SET_ID (V4L2_CID_STATELESS_HEVC_SLICE_PARAMS),
       .size = sizeof(struct v4l2_ctrl_hevc_slice_params),
       .optional = TRUE,
+    }, {
+      SET_ID (V4L2_CID_STATELESS_HEVC_EXT_SPS_ST_RPS),
+      .size = sizeof(struct v4l2_ctrl_hevc_ext_sps_st_rps),
+      .optional = TRUE,
+    }, {
+      SET_ID (V4L2_CID_STATELESS_HEVC_EXT_SPS_LT_RPS),
+      .size = sizeof(struct v4l2_ctrl_hevc_ext_sps_lt_rps),
+      .optional = TRUE,
     }
   };
   #undef SET_ID
@@ -248,6 +258,18 @@ gst_v4l2_codec_h265_dec_open (GstVideoDecoder * decoder)
       .size = sizeof (self->scaling_matrix),
     },
   };
+  struct v4l2_ext_control long_short_term_rps[] = {
+    {
+      .id = V4L2_CID_STATELESS_HEVC_EXT_SPS_ST_RPS,
+      .ptr = self->ext_sps_st_rps->data,
+      .size = sizeof (struct v4l2_ctrl_hevc_ext_sps_st_rps),
+    },
+    {
+      .id = V4L2_CID_STATELESS_HEVC_EXT_SPS_LT_RPS,
+      .ptr = self->ext_sps_lt_rps->data,
+      .size = sizeof (struct v4l2_ctrl_hevc_ext_sps_lt_rps),
+    },
+  };
   /* *INDENT-ON* */
 
   if (!gst_v4l2_decoder_open (self->decoder)) {
@@ -277,6 +299,10 @@ gst_v4l2_codec_h265_dec_open (GstVideoDecoder * decoder)
       gst_v4l2_decoder_query_control_size (self->decoder,
       V4L2_CID_STATELESS_HEVC_ENTRY_POINT_OFFSETS, NULL);
 
+  self->support_long_short_term_rps =
+      gst_v4l2_decoder_get_controls (self->decoder, long_short_term_rps,
+      G_N_ELEMENTS (long_short_term_rps));
+
   self->decode_mode = control[0].value;
   self->start_code = control[1].value;
 
@@ -484,6 +510,7 @@ gst_v4l2_codec_h265_dec_decide_allocation (GstVideoDecoder * decoder,
   GstV4l2CodecH265Dec *self = GST_V4L2_CODEC_H265_DEC (decoder);
   GstCaps *caps = NULL;
   guint min = 0, num_bitstream;
+  gboolean has_videometa;
 
   if (self->streaming)
     goto no_internal_changes;
@@ -492,7 +519,7 @@ gst_v4l2_codec_h265_dec_decide_allocation (GstVideoDecoder * decoder,
   g_clear_object (&self->src_allocator);
   g_clear_object (&self->sink_allocator);
 
-  self->has_videometa = gst_query_find_allocation_meta (query,
+  has_videometa = gst_query_find_allocation_meta (query,
       GST_VIDEO_META_API_TYPE, NULL);
 
   gst_query_parse_allocation (query, &caps, NULL);
@@ -501,10 +528,40 @@ gst_v4l2_codec_h265_dec_decide_allocation (GstVideoDecoder * decoder,
     return FALSE;
   }
 
-  if (gst_video_is_dma_drm_caps (caps) && !self->has_videometa) {
-    GST_ERROR_OBJECT (self,
-        "DMABuf caps negotiated without the mandatory support of VideoMeta");
-    return FALSE;
+  if (gst_video_is_dma_drm_caps (caps)) {
+    if (!has_videometa) {
+      GST_ERROR_OBJECT (self,
+          "DMABuf caps negotiated without the mandatory support of VideoMeta");
+      return FALSE;
+    }
+
+    if (self->need_crop) {
+      GST_ERROR_OBJECT (self,
+          "Frame cropping is not supported when DMABuf caps is negotiated.");
+      return FALSE;
+    }
+  }
+
+  /* Check if we can zero-copy buffers */
+  if (!has_videometa) {
+    GstVideoInfo ref_vinfo;
+    gint i;
+
+    gst_video_info_set_format (&ref_vinfo,
+        GST_VIDEO_INFO_FORMAT (&self->vinfo_drm.vinfo), self->display_width,
+        self->display_height);
+
+    for (i = 0; i < GST_VIDEO_INFO_N_PLANES (&self->vinfo_drm.vinfo); i++) {
+      if (self->vinfo_drm.vinfo.stride[i] != ref_vinfo.stride[i] ||
+          self->vinfo_drm.vinfo.offset[i] != ref_vinfo.offset[i]) {
+        GST_WARNING_OBJECT (self,
+            "GstVideoMeta support required, copying frames.");
+        self->copy_frames = TRUE;
+        break;
+      }
+    }
+  } else {
+    self->copy_frames = self->need_crop;
   }
 
   if (gst_query_get_n_allocation_pools (query) > 0)
@@ -873,6 +930,57 @@ gst_v4l2_codec_h265_dec_fill_slice_params (GstV4l2CodecH265Dec * self,
   }
 }
 
+static void
+gst_v4l2_codec_h265_dec_fill_ext_sps_rps (GstV4l2CodecH265Dec * self,
+    const GstH265SPS * sps)
+{
+  GstH265Decoder *decoder = (GstH265Decoder *) self;
+  struct v4l2_ctrl_hevc_ext_sps_st_rps *ext_sps_st_rps_set;
+  struct v4l2_ctrl_hevc_ext_sps_lt_rps *ext_sps_lt_rps_set;
+  const GstH265SPSEXT *sps_ext = gst_h265_decoder_get_sps_ext (decoder, sps);
+  gint i, j;
+
+  for (i = 0; i < sps->num_short_term_ref_pic_sets; i++) {
+    ext_sps_st_rps_set = &g_array_index (self->ext_sps_st_rps,
+        struct v4l2_ctrl_hevc_ext_sps_st_rps, i);
+
+    ext_sps_st_rps_set->flags |=
+        sps->short_term_ref_pic_set[i].inter_ref_pic_set_prediction_flag ?
+        V4L2_HEVC_EXT_SPS_ST_RPS_FLAG_INTER_REF_PIC_SET_PRED : 0;
+    ext_sps_st_rps_set->delta_idx_minus1 =
+        sps->short_term_ref_pic_set[i].delta_idx_minus1;
+    ext_sps_st_rps_set->delta_rps_sign =
+        sps->short_term_ref_pic_set[i].delta_rps_sign;
+    ext_sps_st_rps_set->abs_delta_rps_minus1 =
+        sps->short_term_ref_pic_set[i].abs_delta_rps_minus1;
+    ext_sps_st_rps_set->num_negative_pics =
+        sps->short_term_ref_pic_set[i].NumNegativePics;
+    ext_sps_st_rps_set->num_positive_pics =
+        sps->short_term_ref_pic_set[i].NumPositivePics;
+
+    for (j = 0; j < 16; j++) {
+      ext_sps_st_rps_set->used_by_curr_pic |=
+          (!!(sps_ext->
+              short_term_ref_pic_set_ext[i].used_by_curr_pic_flag[j])) << j;
+      ext_sps_st_rps_set->use_delta_flag |=
+          (!!(sps_ext->short_term_ref_pic_set_ext[i].use_delta_flag[j])) << j;
+      ext_sps_st_rps_set->delta_poc_s0_minus1[j] =
+          sps_ext->short_term_ref_pic_set_ext[i].delta_poc_s0_minus1[j];
+      ext_sps_st_rps_set->delta_poc_s1_minus1[j] =
+          sps_ext->short_term_ref_pic_set_ext[i].delta_poc_s1_minus1[j];
+    }
+  }
+
+  for (i = 0; i < sps->num_long_term_ref_pics_sps; i++) {
+    ext_sps_lt_rps_set = &g_array_index (self->ext_sps_lt_rps,
+        struct v4l2_ctrl_hevc_ext_sps_lt_rps, i);
+
+    ext_sps_lt_rps_set->lt_ref_pic_poc_lsb_sps = sps->lt_ref_pic_poc_lsb_sps[i];
+    ext_sps_lt_rps_set->flags = sps->used_by_curr_pic_lt_sps_flag[i] ?
+        V4L2_HEVC_EXT_SPS_LT_RPS_FLAG_USED_LT : 0;
+  }
+}
+
 static guint8
 lookup_dpb_index (struct v4l2_hevc_dpb_entry dpb[16], GstH265Picture * ref_pic)
 {
@@ -949,10 +1057,12 @@ gst_v4l2_codec_h265_dec_new_sequence (GstH265Decoder * decoder,
     self->crop_rect_x = sps->crop_rect_x;
     self->crop_rect_y = sps->crop_rect_y;
 
-    /* conformance_window_flag could be set but with zeroed
-     * parameters so check if we really need to crop */
-    self->need_crop |= self->crop_rect_width != sps->width;
-    self->need_crop |= self->crop_rect_height != sps->height;
+    /*
+     * Conformance_window_flag could be set but with zeroed
+     * parameters so check if we really need to crop. We only need
+     * to crop if the x/y are not zero, otherwise it can be handled by
+     * GstVideoMeta in a zero-copy fashion.
+     */
     self->need_crop |= self->crop_rect_x != 0;
     self->need_crop |= self->crop_rect_y != 0;
   }
@@ -984,6 +1094,7 @@ gst_v4l2_codec_h265_dec_new_sequence (GstH265Decoder * decoder,
   }
 
   gst_v4l2_codec_h265_dec_fill_sequence (self, sps);
+  gst_v4l2_codec_h265_dec_fill_ext_sps_rps (self, sps);
 
   if (negotiation_needed) {
     gst_v4l2_codec_h265_dec_streamoff (self);
@@ -993,28 +1104,6 @@ gst_v4l2_codec_h265_dec_new_sequence (GstH265Decoder * decoder,
     }
   }
 
-  /* Check if we can zero-copy buffers */
-  if (!self->has_videometa) {
-    GstVideoInfo ref_vinfo;
-    gint i;
-
-    gst_video_info_set_format (&ref_vinfo,
-        GST_VIDEO_INFO_FORMAT (&self->vinfo_drm.vinfo), self->display_width,
-        self->display_height);
-
-    for (i = 0; i < GST_VIDEO_INFO_N_PLANES (&self->vinfo_drm.vinfo); i++) {
-      if (self->vinfo_drm.vinfo.stride[i] != ref_vinfo.stride[i] ||
-          self->vinfo_drm.vinfo.offset[i] != ref_vinfo.offset[i]) {
-        GST_WARNING_OBJECT (self,
-            "GstVideoMeta support required, copying frames.");
-        self->copy_frames = TRUE;
-        break;
-      }
-    }
-  } else {
-    self->copy_frames = self->need_crop;
-  }
-
   return GST_FLOW_OK;
 }
 
@@ -1118,8 +1207,10 @@ gst_v4l2_codec_h265_dec_start_picture (GstH265Decoder * decoder,
 
   /* The base class will only emit new_sequence for allocation related changes
    * in the SPS, make sure to keep the SPS upt-to-date */
-  if (slice->header.pps->sps->id != self->sps.seq_parameter_set_id)
+  if (slice->header.pps->sps->id != self->sps.seq_parameter_set_id) {
     gst_v4l2_codec_h265_dec_fill_sequence (self, slice->header.pps->sps);
+    gst_v4l2_codec_h265_dec_fill_ext_sps_rps (self, slice->header.pps->sps);
+  }
 
   gst_v4l2_codec_h265_dec_fill_pps (self, slice->header.pps);
   gst_v4l2_codec_h265_dec_fill_scaling_matrix (self, slice->header.pps);
@@ -1266,6 +1357,8 @@ gst_v4l2_codec_h265_dec_output_picture (GstH265Decoder * decoder,
   GstCodecPicture *codec_picture = GST_CODEC_PICTURE (picture);
   gint ret;
 
+  g_return_val_if_fail (request, GST_FLOW_ERROR);
+
   if (codec_picture->discont_state) {
     if (!gst_video_decoder_negotiate (vdec)) {
       GST_ERROR_OBJECT (vdec, "Could not re-negotiate with updated state");
@@ -1371,6 +1464,8 @@ gst_v4l2_codec_h265_dec_submit_bitstream (GstV4l2CodecH265Dec * self,
     { }, /* SLICE_PARAMS */
     { }, /* SCALING_MATRIX */
     { }, /* ENTRY_POINT_OFFSETS */
+    { }, /* EXT_SPS_ST_RPS */
+    { }, /* EXT_SPS_LT_RPS */
   };
   /* *INDENT-ON* */
 
@@ -1392,8 +1487,10 @@ gst_v4l2_codec_h265_dec_submit_bitstream (GstV4l2CodecH265Dec * self,
         system_frame_number);
     g_return_val_if_fail (frame, FALSE);
 
-    if (!gst_v4l2_codec_h265_dec_ensure_output_buffer (self, frame))
+    if (!gst_v4l2_codec_h265_dec_ensure_output_buffer (self, frame)) {
+      gst_video_codec_frame_unref (frame);
       goto done;
+    }
 
     request = gst_v4l2_decoder_alloc_request (self->decoder,
         system_frame_number, self->bitstream, frame->output_buffer);
@@ -1413,6 +1510,24 @@ gst_v4l2_codec_h265_dec_submit_bitstream (GstV4l2CodecH265Dec * self,
     control[num_controls].size = sizeof (self->sps);
     num_controls++;
     self->need_sequence = FALSE;
+    if (self->sps.num_short_term_ref_pic_sets
+        && self->support_long_short_term_rps) {
+      control[num_controls].id = V4L2_CID_STATELESS_HEVC_EXT_SPS_ST_RPS;
+      control[num_controls].ptr = self->ext_sps_st_rps->data;
+      control[num_controls].size =
+          g_array_get_element_size (self->ext_sps_st_rps)
+          * self->sps.num_short_term_ref_pic_sets;
+      num_controls++;
+    }
+    if (self->sps.num_long_term_ref_pics_sps
+        && self->support_long_short_term_rps) {
+      control[num_controls].id = V4L2_CID_STATELESS_HEVC_EXT_SPS_LT_RPS;
+      control[num_controls].ptr = self->ext_sps_lt_rps->data;
+      control[num_controls].size =
+          g_array_get_element_size (self->ext_sps_lt_rps)
+          * self->sps.num_long_term_ref_pics_sps;
+      num_controls++;
+    }
   }
 
   if (self->first_slice) {
@@ -1668,6 +1783,12 @@ gst_v4l2_codec_h265_dec_init (GstV4l2CodecH265Dec * self,
   g_array_set_size (self->slice_params, 4);
   self->entry_point_offsets = g_array_sized_new (FALSE, TRUE,
       sizeof (guint32), 4);
+  self->ext_sps_st_rps = g_array_sized_new (FALSE, TRUE,
+      sizeof (struct v4l2_ctrl_hevc_ext_sps_st_rps), 65);
+  g_array_set_size (self->ext_sps_st_rps, 65);
+  self->ext_sps_lt_rps = g_array_sized_new (FALSE, TRUE,
+      sizeof (struct v4l2_ctrl_hevc_ext_sps_lt_rps), 65);
+  g_array_set_size (self->ext_sps_lt_rps, 65);
 }
 
 static void
@@ -1676,6 +1797,8 @@ gst_v4l2_codec_h265_dec_dispose (GObject * object)
   GstV4l2CodecH265Dec *self = GST_V4L2_CODEC_H265_DEC (object);
 
   g_clear_object (&self->decoder);
+  g_clear_pointer (&self->ext_sps_st_rps, g_array_unref);
+  g_clear_pointer (&self->ext_sps_lt_rps, g_array_unref);
   g_clear_pointer (&self->slice_params, g_array_unref);
   g_clear_pointer (&self->entry_point_offsets, g_array_unref);
 
